* {
    font-family:"Helvetica Neue", Helvetica, Arial;
    margin: 0;
    padding: 0;
}

h1 {
    padding: 10px 0 10px 10px;
    color: white;
    background-color: #e68a00;
}

a.title {
    text-decoration:none !important;
}

h2 {
    background-color: #f2f2f2;
    padding: 10px;
}

.task-info h3 {
    padding: 25px 0 10px 0;
}


.task-queue {
    width: 90%;
    border-collapse: collapse;

}

.container {
    margin-top: 10px;
    margin-left:100px;
}


.task-queue th {
    color: white;
    background-color: rgb(41, 128, 185);
    font-size: 14px;
    padding:6px 12px;
    text-align: left;

}

.failed th {
    background-color: rgb(234, 97, 83) !important;
}

.completed th {
    background-color: rgb(39, 174, 96) !important;
}

.task-queue td {
    padding:6px 12px;
    -webkit-font-smoothing: antialiased;
    font-smoothing: antialiased;
    color: #3b3b3b;
}

.task-queue tr:nth-of-type(odd) {
    background-color: rgb(233, 233, 233);
}

.task-queue tr:hover {
    cursor: pointer;
    background-color: #96c7e8;
}
.strapline {
    margin: 10px 0;
}


.traceback {
    list-style-type: none;
    font-family: "Courier New", Courier, monospace !important;
    width: 90%;
    margin-top: 10px;
    padding: 10px;
    font-size: 14px;
}

.tb {
    font-family: "Courier New", Courier, monospace !important;
    padding-bottom: 5px;
}

.indent_level_1 {
    padding-left: 25px;
}

.indent_level_2 {
    padding-left: 50px;
    padding-bottom: 25px;
}

.yellow {
    background-color: lightyellow;
    border: solid 1px #ffdb99;
    background-color: lightyellow;
}

.green {
    background-color: #d9f2d9;
    border: solid 1px lightgreen;
}


.left {
    margin: 10px 0 !important;
}

li.DEBUG {
    color: grey;
}

li.INFO {
    color: green;
}

li.WARNING {
    color: orange;
}

li.ERROR {
    color: red;
}

li.CRITICAL {
    color: red;
    font-weight: bold;
}

.levelselect {
    list-style-type: none;
    height: 15px;
    width: 300px;
    padding: 5px 15px;
    margin: 10px;
    background: lightgrey;
    border: solid 1px grey;
    border-radius: 3px;

}

.levelselect li {
    float: left;
    margin-left: 10px;
    font-weight: bold;
}

.levelselect li a {
    text-decoration: none;
    color: black;
    font-weight: normal;
}

.paginator {
    margin: 10px 0;
}


.paginator li {
    display: inline-block;
    list-style-type: none;
    vertical-align: middle;
}

button.page {
    color: rgb(41, 128, 185);
    background-color: white;
    float: left;
    padding: 3px;
    border: none;
    transition: .3s;
}

button.page:disabled, button.page:disabled:hover {
    color: grey;
    background-color: white;
    border: none;

}

button.page:hover {
    background-color: rgb(41, 128, 185);
    border-radius: 5px;
    color: white;
}


#overlay {
    position: fixed;
    overflow: hidden;
    width:100%;
    top: 0px;
    height:100%;
    z-index: 1;
    background: #000000;
    opacity: 0.7;
}

body {
    position: absolute;
    overflow: scroll;
    left: 0;
    right:0;
    top: 0;
    bottom: 0;
}


.task-view {
	background-color: white !important;
	position: absolute;
	padding: 20px 0 0 65px;
	width: auto;
	min-width: 800px;
    min-height: 90%;
    overflow-y: scroll;
    top: 20px;
    right: 300px;
    left: 300px;
    bottom: 10px;
    z-index: 10000;
}

.task-view h2 {
    background-color:white !important;
    padding: 0;
}

.task-view h3 {
    margin-top: 25px;
}

.task-view .task-queue tr:hover {
    cursor: auto;
    background-color: white;
}

.task-view .task-queue tr:nth-of-type(odd) {
    background-color: rgb(233, 233, 233);
}

.buttonlist {
    margin: 20px;
}

.buttonlist button {
    background-color: rgb(41, 128, 185);
    border: none;
    padding: 10px;
    color: white;
    transition-duration: 0.3s;
    cursor: pointer;
    font-size: 14px;
    border: solid 1px white;
}

.buttonlist button:hover {
    color: rgb(41, 128, 185);
    background-color: white;
    border: solid 1px rgb(41, 128, 185);
}

.buttonlist button.delete {
    background-color: rgb(234, 97, 83);
}

.buttonlist button.delete:hover {
    color: rgb(234, 97, 83);
    background-color: white;
    border: solid 1px rgb(234, 97, 83);
}

#save-msg {
    padding-left: 5px;
    font-size: 14px;
}

#save-msg.success {
    color: green;
}

#save-msg.failure {
    color: red;
}

.fade-enter-active, .fade-leave-active {
  transition: opacity .5s
}

.fade-enter, .fade-leave-to {
  opacity: 0
}

.field_error {
    list-style-type: none;
    color: red;
}

.field_error li {
    padding: 5px !important;
    margin: 0 !important;
}

.error_td {
    background-color: #ffe6e6;
    width: 325px;
}

.task-info {
    width: 90%;
    border-collapse: collapse;
    padding: 10px 50px;

}

input[type=text], input[type=number] {
    width: 300px;
    border: 2px solid lightgrey;
    padding: 10px;
    outline: none;
    border-radius: 15px;
    font-size: 14px;
    transition-duration: 0.5s;
}


input:focus {
    border: 2px solid rgb(41, 128, 185);
    box-shadow: 3px 3px 5px grey;
}

select {
    margin-top: 3px;
    width: 320px;
    border: 2px solid lightgrey;
    border-radius: 15px;
    padding: 10px 0 10px 10px;
    box-shadow: none;
    background: transparent;
    font-size: 14px;
    background: white url("ic_keyboard_arrow_down_black_24px.svg") no-repeat 95% 80%;
    -webkit-appearance: none;
    transition-duration: 0.5s;
}

select:hover {
    cursor: pointer;
}

select:focus {
    border: 2px solid rgb(41, 128, 185);
    box-shadow: 3px 3px 5px grey;
}

option {
    padding: 50px;
}

.task-form {
    list-style-type: none;
}

.task-form label {
    display: block;
}

.task-form li {
    margin: 10px 5px;
    border-radius: 5px;
    padding: 0 10px;

}

textarea {
    margin-top: 3px;
    resize: none;
    padding: 10px;
    border: 2px solid lightgrey;
    border-radius: 15px;
    width: 300px;
    height: 100px;
    outline: none;
    transition-duration: 0.5s
}

textarea:focus {
    border: 2px solid rgb(41, 128, 185);
    box-shadow: 3px 3px 5px grey;
}

.checkbox > input[type=checkbox] {
    visibility: hidden;
}


.checkbox {
   position: relative;
   display: block;
   width: 80px;
   height: 26px;
   margin: 0;
   background: #FFF;
   border: 2px solid lightgrey;
   border-radius: 5px;
   -webkit-border-radius: 5px;
   -moz-border-radius: 5px;
}

.checkbox:after {
   position: absolute;
   display: inline;
   right: 10px;
   content: 'No';
   color: #E53935;
   font: 12px/26px Arial, sans-serif;
   font-weight: bold;
   z-index: 0;
}

.checkbox:before {
   position: absolute;
   display: inline;
   left: 10px;
   content: 'Yes';
   color: #43A047;
   font: 12px/26px Arial, sans-serif;
   font-weight: bold;
   z-index: 0;
}

.checkbox label {
   position: absolute;
   display: block;
   top: 3px;
   left: 3px;
   width: 34px;
   height: 20px;
   background-color: rgb(41, 128, 185);
   cursor: pointer;
   transition: all 0.5s linear;
   -webkit-transition: all 0.5s linear;
   -moz-transition: all 0.5s linear;
   border-radius: 2px;
   -webkit-border-radius: 2px;
   -moz-border-radius: 2px;
   z-index: 1;
}

.checkbox input[type=checkbox]:checked + label {
   left: 43px;
}


input[type=number]::-webkit-inner-spin-button,
input[type=number]::-webkit-outer-spin-button {
  -webkit-appearance: none;
  margin: 0;
}

.breadcrumbs {
    width: 100%;
    list-style-type: none;
    display: inline-block;
    background: rgb(100,100,100);
    margin: 0;
}

.breadcrumbs li {
    color: white;
    float: left;
    padding: 5px;
    cursor: pointer;

}

.searchbar {
    margin-bottom: 5px;
    width: 90%;
    padding: 5px;
}